@@ -5,7 +5,7 @@
 
  */
 
-#define VERSION "1.3.0"
+#include "version.h"
 
 void draw_desktop(void);
 void colorwin(WINDOW * win);
@@ -21,3 +21,4 @@ void listkeyhelp();
 void indicate(char *message);
 void printlargenum(unsigned long long i, WINDOW *win);
 void standardcolors(int color);
+void refresh_screen(void);
