@@ -15,15 +15,15 @@
  */
  
 #ifndef WORKDIR
-#define WORKDIR		"/var/local/iptraf/"
+#define WORKDIR		"/var/local/iptraf"
 #endif
 
 #ifndef LOGDIR
-#define LOGDIR		"/var/log/iptraf/"
+#define LOGDIR		"/var/log/iptraf"
 #endif
 
 #ifndef EXECDIR
-#define EXECDIR		"/usr/local/bin/"
+#define EXECDIR		"/usr/local/bin"
 #endif
 
 
@@ -108,7 +108,7 @@
 #define TCPUDPLOG	get_path(T_LOGDIR, "tcp_udp_services.log")
 #define LANLOG		get_path(T_LOGDIR, "lan_statistics.log")
 #define PKTSIZELOG	get_path(T_LOGDIR, "packet_size.log")
-
+#define DAEMONLOG	get_path(T_LOGDIR, "daemon.log")
 /*
  * The additional TCP/UDP ports file
  */
@@ -139,3 +139,7 @@
  */
 
 #define TCPFILTERSAVE   get_path(T_WORKDIR, "tcpfilter.cur")
+
+#ifndef PATH_MAX
+#define PATH_MAX	4095
+#endif
