@@ -22,6 +22,8 @@ details.
 #include <panel.h>
 #include "deskman.h"
 #include "attrs.h"
+#include "error.h"
+#include "log.h"
 
 void errbox(char *message, char *prompt, int *response)
 {
@@ -41,10 +43,25 @@ void errbox(char *message, char *prompt, int *response)
     wprintw(win, "%s", message);
     update_panels();
     doupdate();
-    *response = wgetch(win);
+
+    do {
+	*response = wgetch(win);
+	if (*response == 12)
+	    refresh_screen();
+    } while (*response == 12);
 
     del_panel(panel);
     delwin(win);
     update_panels();
     doupdate();
 }
+
+void write_error(char *msg, int daemonized)
+{
+    int response;
+
+    if (daemonized)
+	write_daemon_err(msg);
+    else
+	errbox(msg, ANYKEY_MSG, &response);
+}
