@@ -31,5 +31,6 @@ struct ffnode {
 };
 
 int loadfilterlist(unsigned int protocol, struct ffnode **fltfile);
+void save_filterlist(unsigned int protocol, struct ffnode *fltlist);
 void destroyfilterlist(struct ffnode *fltlist);
-
+void get_filter_description(char *description, int *aborted, char *pre_edit);
