@@ -6,4 +6,7 @@ ifaces.h - prototype declaration for interface support determination
 ***/
 
 int iface_supported(char *iface);
+int iface_up(char *iface);
 void err_iface_unsupported(void);
+void err_iface_down(void);
+void isdn_iface_check(int *fd, char *ifname);
