@@ -57,6 +57,9 @@ details.
 #define SCROLLUP 0
 #define SCROLLDOWN 1
 
+extern int exitloop;
+extern int daemonized;
+
 /* from log.c, applicable only to this module */
 
 void writegstatlog(struct iftab *table, int unit, unsigned long nsecs, FILE * logfile);
@@ -410,7 +413,7 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 
     struct isdntab isdntable;
 
-    char buf[8192];
+    char buf[MAX_PACKET_SIZE];
     char aligned_buf[120];
     char *packet;
     struct iphdr *ipacket;
@@ -435,7 +438,6 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 
     int ch;
 
-    int endloop = 0;
     struct timeval tv;
     unsigned long starttime = 0;
     unsigned long statbegin = 0;
@@ -448,8 +450,8 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
     if (!facility_active(GSTATIDFILE))
 	mark_facility(GSTATIDFILE, "general interface statistics");
     else {
-	errbox("General interface stats already active in another process",
-	       ANYKEY_MSG, &ch);
+	write_error("General interface stats already active in another process",
+		    daemonized);
 	return;
     }
 
@@ -481,10 +483,11 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 
 	bzero(&isdntable, sizeof(struct isdntab));
 
+	exitloop = 0;
 	gettimeofday(&tv, NULL);
 	starttime = startlog = statbegin = tv.tv_sec;
 
-	while (!endloop) {
+	while (!exitloop) {
 	    getpacket(fd, buf, &fromaddr, &ch, &br, ifname, table.statwin);
 
 	    if (ch != ERR) {
@@ -503,13 +506,19 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 		case ' ':
 		    pagegstatwin(&table, SCROLLUP, &idx);
 		    break;
+		case 12:
+		case 'l':
+		case 'L':
+		    refresh_screen();
+		    break;
+
 		case 'Q':
 		case 'q':
 		case 'X':
 		case 'x':
 		case 27:
 		case 24:
-		    endloop = 1;
+		    exitloop = 1;
 		    break;
 		}
 	    }
@@ -574,7 +583,7 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 	    check_rotate_flag(&logfile, logging);
 
 	    if ((facilitytime != 0) && (((now - statbegin) / 60) >= facilitytime))
-		endloop = 1;
+		exitloop = 1;
 	}
 
 	close(fd);
@@ -696,7 +705,7 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     WINDOW *statwin;
     PANEL *statpanel;
 
-    char buf[8192];
+    char buf[MAX_PACKET_SIZE];
     char aligned_buf[120];
     char *packet;
     struct iphdr *ipacket = NULL;
@@ -725,7 +734,6 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     struct timeval tv;
     unsigned long updtime = 0;
     unsigned long long updtime_usec = 0;
-    int endloop = 0;
     unsigned long starttime, now;
     unsigned long statbegin, startlog;
     unsigned long long unow;
@@ -760,8 +768,8 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     if (!facility_active(DSTATIDFILE))
 	mark_facility(DSTATIDFILE, "detailed interface statistics");
     else {
-	errbox("Detailed interface stats already active in another process",
-	       ANYKEY_MSG, &ch);
+	write_error("Detailed interface stats already active in another process",
+		    daemonized);
 	return;
     }
 
@@ -824,24 +832,31 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 
     leaveok(statwin, TRUE);
 
+    exitloop = 0;
     dispmode(options->actmode, unitstring);
 
     /*
      * Data-gathering loop
      */
 
-    while (!endloop) {
+    while (!exitloop) {
 	getpacket(fd, buf, &fromaddr, &ch, &br, ifname, statwin);
 
 	if (ch != ERR) {
 	    switch (ch) {
+	    case 12:
+	    case 'l':
+	    case 'L':
+		refresh_screen();
+		break;
+
 	    case 'Q':
 	    case 'q':
 	    case 'X':
 	    case 'x':
 	    case 24:
 	    case 27:
-		endloop = 1;
+		exitloop = 1;
 		break;
 	    }
 	}
@@ -1035,7 +1050,7 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 	check_rotate_flag(&logfile, logging);
 
 	if ((facilitytime != 0) && (((now - statbegin) / 60) >= facilitytime))
-	    endloop = 1;
+	    exitloop = 1;
     }
 
     close(fd);
@@ -1058,7 +1073,7 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     doupdate();
 }
 
-void selectiface(char *ifname, int *aborted)
+void selectiface(char *ifname, int withall, int *aborted)
 {
     WINDOW *win;
     PANEL *panel;
@@ -1067,13 +1082,22 @@ void selectiface(char *ifname, int *aborted)
 
     int row;
     int ch;
-    int exitloop = 0;
+
+    int endloop = 0;
 
     struct iflist *list;
     struct iflist *ptmp;
 
     initiflist(&list);
+    if (withall) {
+	ptmp = malloc(sizeof(struct iflist));
+	strcpy(ptmp->ifname, "All Interfaces");
 
+	ptmp->prev_entry = NULL;
+	list->prev_entry = ptmp;
+	ptmp->next_entry = list;
+	list = ptmp;
+    }
     listkeyhelp();
     bw = newwin(14, 24, (LINES - 14) / 2, (COLS - 24) / 2 - 9);
     /* I'm guessing (COLS-24)/2-9*COLS/80) is overkill */
@@ -1146,26 +1170,34 @@ void selectiface(char *ifname, int *aborted)
 	    }
 	    break;
 	case 13:
-	    exitloop = 1;
+	    endloop = 1;
 	    *aborted = 0;
 	    break;
+	case 12:
+	case 'l':
+	case 'L':
+	    refresh_screen();
+	    break;
 	case 27:
 	case 24:
 	case 'X':
 	case 'x':
 	case 'Q':
 	case 'q':
-	    exitloop = 1;
+	    endloop = 1;
 	    *aborted = 1;
 	    break;
 	}
 	update_panels();
 	doupdate();
-    } while (!exitloop);
-
-    if (!(*aborted))
-	strcpy(ifname, ptmp->ifname);
+    } while (!endloop);
 
+    if (!(*aborted)) {
+	if ((withall) && (ptmp->prev_entry == NULL))	/* All Interfaces */
+	    strcpy(ifname, "");
+	else
+	    strcpy(ifname, ptmp->ifname);
+    }
     destroyiflist(list);
     del_panel(panel);
     delwin(win);
