@@ -115,6 +115,9 @@ void getinput(struct FIELDLIST *list, struct FIELD *field, int *exitkey)
 	    endloop = 1;
 	    *exitkey = ch;
 
+	    break;
+	case 12:
+	    refresh_screen();
 	    break;
 	default:
 	    if ((field->tlen < field->len) && ((ch >= 32) && (ch <= 127))) {
