@@ -6,7 +6,7 @@ An IP Network Statistics Utility
 Written by Gerard Paul Java <riker@mozcom.com>
 Copyright (c) Gerard Paul Java 1997-1999
    	
-Version 2.0.0
+Version 2.1.1
 Main Module
 
 This software is open-source; you can redistribute it and/or modify
@@ -46,177 +46,18 @@ details.
 #include "attrs.h"
 #include "error.h"
 
+#define WITHALL 1
+#define WITHOUTALL 0
+
+#define ALLSPEC "all"
+
+int exitloop = 0;
+int daemonized = 0;
+
 extern void srpromisc(int mode);
 extern void about();
 extern void masq_warning();
 
-int check_ip_masquerade(void)
-{
-    int fd;
-
-    fd = open("/proc/net/ip_masquerade", O_RDONLY);
-
-    if (fd >= 0) {
-	close(fd);
-	return 1;
-    }
-    return 0;
-}
-
-void init_break_menu(struct MENU *break_menu)
-{
-    initmenu(break_menu, 5, 20, (LINES - 5) / 2, COLS / 2);
-    additem(break_menu, " By packet ^s^ize", "Displays packet counts by packet size range");
-    additem(break_menu, " By TCP/UDP ^p^ort", "Displays packet and byte counts by service port");
-    additem(break_menu, " E^x^it menu", "Return to main menu");
-}
-
-/*
- * Get the ball rolling: The program interface routine.
- */
-
-void program_interface(struct OPTIONS *options,
-		       int opt, int dontwarn, char *optarg,
-		       int facilitytime, int is_first_instance)
-{
-    struct MENU menu;
-    struct MENU break_menu;
-
-    int endloop = 0;
-    int row = 1;
-    int break_row = 1;
-    int aborted;
-    int break_aborted;
-    int faborted;
-    int filtered = 0;
-    struct filterlist fl;
-    struct othpoptions ofilter;
-    char ifname[10];
-    struct porttab *ports;
-
-    draw_desktop();
-    attrset(STDATTR);
-    move(0, 1);
-    printw("IPTraf");
-
-    loadsavedtcpfilter(&fl, &filtered);
-    loadothfilter(&ofilter);
-    indicate("");
-    loadaddports(&ports);
-
-    if ((!dontwarn) && (check_ip_masquerade()))
-	masq_warning();
-
-    if (opt == 0) {
-	about();
-
-	initmenu(&menu, 14, 35, (LINES - 14) / 2, (COLS - 35) / 2);
-
-	additem(&menu, " IP traffic ^m^onitor", "Displays current IP traffic information");
-	additem(&menu, " General interface ^s^tatistics", "Displays some statistics for attached interfaces");
-	additem(&menu, " ^D^etailed interface statistics", "Displays more statistics for a selected interface");
-	additem(&menu, " Statistical ^b^reakdowns", "Facilities for traffic counts by port, packet size, or address");
-	additem(&menu, " ^L^AN station monitor", "Displays statistics on detected LAN stations");
-	additem(&menu, NULL, NULL);
-	additem(&menu, " ^T^CP display filters", "Manages TCP display filters");
-	additem(&menu, " Other ^p^rotocol filters", "Select which non-TCP protocols to display");
-	additem(&menu, NULL, NULL);
-	additem(&menu, " C^o^nfigure", "Set various program options");
-	additem(&menu, NULL, NULL);
-	additem(&menu, " E^x^it", "Exits program");
-
-	endloop = 0;
-
-	do {
-	    showmenu(&menu);
-	    operatemenu(&menu, &row, &aborted);
-
-	    switch (row) {
-	    case 1:
-		ipmon(options,
-		      filtered, &fl, &ofilter, options->timeout, 0);
-		break;
-	    case 2:
-		ifstats(options, 0);
-		break;
-	    case 3:
-		selectiface(ifname, &aborted);
-		if (!aborted)
-		    detstats(ifname, options, 0);
-		break;
-	    case 4:
-		break_row = 1;
-		init_break_menu(&break_menu);
-		showmenu(&break_menu);
-		operatemenu(&break_menu, &break_row, &break_aborted);
-
-		switch (break_row) {
-		case 1:
-		    selectiface(ifname, &aborted);
-		    if (!aborted)
-			packet_size_breakdown(options, ifname, 0);
-		    break;
-		case 2:
-		    selectiface(ifname, &aborted);
-		    if (!aborted)
-			servmon(ifname, ports, options, 0);
-		    break;
-		case 4:
-		    break;
-		}
-		destroymenu(&break_menu);
-		break;
-	    case 5:
-		hostmon(options, 0);
-		break;
-	    case 7:
-		tcpfilterselect(&fl, &filtered, &faborted);
-		break;
-	    case 8:
-		othfilterselect(&ofilter);
-		saveothfilter(&ofilter);
-		break;
-	    case 10:
-		setoptions(options, &ports, is_first_instance);
-		saveoptions(options);
-		break;
-	    case 12:
-		endloop = 1;
-		break;
-	    }
-	} while (!endloop);
-
-	destroymenu(&menu);
-    } else {
-	switch (opt) {
-	case 'i':
-	    ipmon(options,
-		filtered, &fl, &ofilter, options->timeout, facilitytime);
-	    break;
-	case 'g':
-	    ifstats(options, facilitytime);
-	    break;
-	case 'd':
-	    detstats(optarg, options, facilitytime);
-	    break;
-	case 's':
-	    servmon(optarg, ports, options, facilitytime);
-	    break;
-	case 'z':
-	    packet_size_breakdown(options, optarg, facilitytime);
-	    break;
-	case 'l':
-	    hostmon(options, facilitytime);
-	    break;
-	}
-    }
-
-    destroyporttab(ports);
-    erase();
-    update_panels();
-    doupdate();
-}
-
 void removetags()
 {
     unlink(IPTIDFILE);
@@ -273,6 +114,214 @@ void segvhandler()
     exit(2);
 }
 
+void usr2handler()
+{
+    exitloop = 1;
+}
+
+
+int check_ip_masquerade(void)
+{
+    int fd;
+
+    fd = open("/proc/net/ip_masquerade", O_RDONLY);
+
+    if (fd >= 0) {
+	close(fd);
+	return 1;
+    }
+    return 0;
+}
+
+void init_break_menu(struct MENU *break_menu)
+{
+    initmenu(break_menu, 5, 20, (LINES - 5) / 2, COLS / 2);
+    additem(break_menu, " By packet ^s^ize", "Displays packet counts by packet size range");
+    additem(break_menu, " By TCP/UDP ^p^ort", "Displays packet and byte counts by service port");
+    additem(break_menu, " E^x^it menu", "Return to main menu");
+}
+
+/*
+ * Get the ball rolling: The program interface routine.
+ */
+
+void program_interface(struct OPTIONS *options,
+		       int opt, int dontwarn, char *optarg,
+		       int facilitytime, int is_first_instance)
+{
+    struct MENU menu;
+    struct MENU break_menu;
+
+    int endloop = 0;
+    int row = 1;
+    int break_row = 1;
+    int aborted;
+    int break_aborted;
+    int faborted;
+    int filtered = 0;
+    struct filterlist fl;
+    struct othpoptions ofilter;
+    char ifname[10];
+    char *ifptr = NULL;
+    struct porttab *ports;
+
+    draw_desktop();
+    attrset(STDATTR);
+    move(0, 1);
+    printw("IPTraf");
+
+    loadsavedtcpfilter(&fl, &filtered);
+    loadothfilter(&ofilter);
+    indicate("");
+    loadaddports(&ports);
+
+    /*
+     * Issue masquerading warning onscreen (intaractive mode) or in the log
+     * (daemonized mode).
+     */
+
+    if ((!dontwarn) && (check_ip_masquerade())) {
+	if (!daemonized)
+	    masq_warning();
+	else
+	    write_daemon_err("Warning: IP masquerading enabled, you may get strange results");
+    }
+    if (opt == 0) {
+	about();
+
+	initmenu(&menu, 14, 35, (LINES - 14) / 2, (COLS - 35) / 2);
+
+	additem(&menu, " IP traffic ^m^onitor", "Displays current IP traffic information");
+	additem(&menu, " General interface ^s^tatistics", "Displays some statistics for attached interfaces");
+	additem(&menu, " ^D^etailed interface statistics", "Displays more statistics for a selected interface");
+	additem(&menu, " Statistical ^b^reakdowns", "Facilities for traffic counts by packet size or TCP/UDP port");
+	additem(&menu, " ^L^AN station monitor", "Displays statistics on detected LAN stations");
+	additem(&menu, NULL, NULL);
+	additem(&menu, " ^T^CP display filters", "Manages TCP display filters");
+	additem(&menu, " Other ^p^rotocol filters", "Select which non-TCP protocols to display");
+	additem(&menu, NULL, NULL);
+	additem(&menu, " C^o^nfigure", "Set various program options");
+	additem(&menu, NULL, NULL);
+	additem(&menu, " E^x^it", "Exits program");
+
+	endloop = 0;
+
+	do {
+	    showmenu(&menu);
+	    operatemenu(&menu, &row, &aborted);
+
+	    switch (row) {
+	    case 1:
+		selectiface(ifname, WITHALL, &aborted);
+		if (!aborted) {
+		    if (strcmp(ifname, "") != 0)
+			ifptr = ifname;
+		    else
+			ifptr = NULL;
+
+		    ipmon(options,
+		    filtered, &fl, &ofilter, options->timeout, 0, ifptr);
+		}
+		break;
+	    case 2:
+		ifstats(options, 0);
+		break;
+	    case 3:
+		selectiface(ifname, WITHOUTALL, &aborted);
+		if (!aborted)
+		    detstats(ifname, options, 0);
+		break;
+	    case 4:
+		break_row = 1;
+		init_break_menu(&break_menu);
+		showmenu(&break_menu);
+		operatemenu(&break_menu, &break_row, &break_aborted);
+
+		switch (break_row) {
+		case 1:
+		    selectiface(ifname, WITHOUTALL, &aborted);
+		    if (!aborted)
+			packet_size_breakdown(options, ifname, 0);
+		    break;
+		case 2:
+		    selectiface(ifname, WITHOUTALL, &aborted);
+		    if (!aborted)
+			servmon(ifname, ports, options, 0);
+		    break;
+		case 4:
+		    break;
+		}
+		destroymenu(&break_menu);
+		break;
+	    case 5:
+		selectiface(ifname, WITHALL, &aborted);
+		if (!aborted) {
+		    if (strcmp(ifname, "") != 0)
+			ifptr = ifname;
+		    else
+			ifptr = NULL;
+		    hostmon(options, 0, ifptr);
+		}
+		break;
+	    case 7:
+		tcpfilterselect(&fl, &filtered, &faborted);
+		break;
+	    case 8:
+		othfilterselect(&ofilter);
+		saveothfilter(&ofilter);
+		break;
+	    case 10:
+		setoptions(options, &ports, is_first_instance);
+		saveoptions(options);
+		break;
+	    case 12:
+		endloop = 1;
+		break;
+	    }
+	} while (!endloop);
+
+	destroymenu(&menu);
+    } else {
+	switch (opt) {
+	case 'i':
+	    if ((strcmp(optarg, ALLSPEC) == 0) || (strcmp(optarg, "") == 0))
+		ifptr = NULL;
+	    else
+		ifptr = optarg;
+
+	    ipmon(options,
+		  filtered, &fl, &ofilter, options->timeout, facilitytime, ifptr);
+	    break;
+	case 'g':
+	    ifstats(options, facilitytime);
+	    break;
+	case 'd':
+	    detstats(optarg, options, facilitytime);
+	    break;
+	case 's':
+	    servmon(optarg, ports, options, facilitytime);
+	    break;
+	case 'z':
+	    packet_size_breakdown(options, optarg, facilitytime);
+	    break;
+	case 'l':
+	    if ((strcmp(optarg, ALLSPEC) == 0) || (strcmp(optarg, "") == 0))
+		ifptr = NULL;
+	    else
+		ifptr = optarg;
+
+	    hostmon(options, facilitytime, ifptr);
+	    break;
+	}
+    }
+
+    destroyporttab(ports);
+    erase();
+    update_panels();
+    doupdate();
+}
+
+
 /*
  * Command-line help facility.
  */
@@ -280,24 +329,25 @@ void segvhandler()
 void commandhelp()
 {
     printf("\nSyntax:\n");
-    printf("    iptraf [ -f ][ -q ][ { -i | -g | -d iface | -s iface | -z iface | -l }\n");
-    printf("           [-t timeout ]]\n\n");
+    printf("    iptraf [ -f ][ -q ][ { -i iface | -g | -d iface | -s iface | -z iface |\n");
+    printf("           -l iface } [ -B ] [-t timeout ]]\n\n");
     printf("Issue the iptraf command with no parameters for menu-driven operation.\n");
     printf("These options can also be supplied to the command:\n\n");
-    printf("	-i		- start the IP traffic monitor\n");
-    printf("	-g		- start the general interface statistics\n");
-    printf("	-d iface	- start the detailed statistics facility on an interface\n");
-    printf("	-s iface	- start the TCP and UDP monitor on an interface\n");
-    printf("	-z iface	- shows the packet size counts on an interface\n");
-    printf("	-l		- start the LAN station monitor\n");
-    printf("	-t timeout	- when used with one of the above parameters, tells\n");
-    printf("			  the facility to run only for the specified number of\n");
-    printf("			  minutes (timeout)\n");
-    printf("	-f		- Remove all identification tag files.  This will\n");
-    printf("			  allow multiple instances of facilities in different\n");
-    printf("			  IPTraf processes.  Use with great caution.  May be\n");
-    printf("			  used after an abnormal termination.\n");
-    printf("	-q		- suppress IP masquerading warning on startup\n\n");
+    printf("-i iface    - start the IP traffic monitor (use \"-i all\" for all interfaces)\n");
+    printf("-g          - start the general interface statistics\n");
+    printf("-d iface    - start the detailed statistics facility on an interface\n");
+    printf("-s iface    - start the TCP and UDP monitor on an interface\n");
+    printf("-z iface    - shows the packet size counts on an interface\n");
+    printf("-l iface    - start the LAN station monitor (\"-l all\" for all LAN interfaces)\n");
+    printf("-B          - Run in background (use only with one of the above parameters)\n");
+    printf("-t timeout  - when used with one of the above parameters, tells\n");
+    printf("              the facility to run only for the specified number of\n");
+    printf("              minutes (timeout)\n");
+    printf("-f          - Remove all lock files.  This will allow multiple\n");
+    printf("              instances of facilities in different IPTraf processes.\n");
+    printf("              Use with great caution.  Normally used to recover from\n");
+    printf("              an abnormal termination.\n");
+    printf("-q          - suppress IP masquerading warning on startup\n\n");
     printf("IPTraf %s Copyright (c) Gerard Paul Java 1997-1999\n", VERSION);
 }
 
@@ -363,7 +413,7 @@ int main(int argc, char **argv)
     if (argc > 1) {
 	do {
 	    opterr = 0;
-	    opt = getopt(argc, argv, "igd:s:z:lhfqt:");
+	    opt = getopt(argc, argv, "i::gd:s:z:l::hfqt:B");
 
 	    if (opt == 'h') {
 		commandhelp();
@@ -378,14 +428,18 @@ int main(int argc, char **argv)
 		}
 	    } else if (opt == 'q') {
 		dontwarn = 1;
+	    } else if (opt == 'B') {	/* daemonization sequence */
+		daemonized = 1;
 	    } else if (opt == '?') {
-		fprintf(stderr, "\nInvalid option or missing parameter\n\n");
+		fprintf(stderr, "\nInvalid option or missing parameter, use iptraf -h for help\n\n");
 		exit(1);
 	    } else if (opt != -1) {
 		if (optarg != 0) {
 		    bzero(keyparm, 12);
 		    strncpy(keyparm, optarg, 11);
-		}
+		} else
+		    strcpy(keyparm, "");
+		    
 		command = opt;
 	    }
 	} while ((opt != '?') && (opt != -1));
@@ -399,6 +453,35 @@ int main(int argc, char **argv)
     }
     loadoptions(&options);
 
+    /*
+     * If a facility is directly invoked from the command line, check for
+     * a daemonization request
+     */
+
+    if ((daemonized) && (command != 0)) {
+	switch (fork()) {
+	case 0:		/* child */
+	    setsid();
+	    freopen("/dev/null", "w", stdout);	/* redirect std output */
+	    freopen("/dev/null", "r", stdin);
+	    freopen("/dev/null", "w", stderr);
+	    signal(SIGUSR2, (void *) usr2handler);
+	    options.logging = 1;               /* daemon mode implies logging */
+	    break;
+	case -1:		/* error */
+	    fprintf(stderr, "\nFork error, IPTraf cannot run in background\n\n");
+	    exit(1);
+	default:		/* parent */
+	    exit(0);
+	}
+    }
+#ifdef SIMDAEMON
+    daemonized = 1;
+    freopen("/dev/null", "w", stdout);	/* redirect std output */
+    freopen("/dev/null", "r", stdin);
+    freopen("/dev/null", "w", stderr);
+#endif
+
     initscr();
 
     mark_first_instance();
