@@ -45,6 +45,9 @@ details.
 #define SCROLLUP 0
 #define SCROLLDOWN 1
 
+extern int exitloop;
+extern int daemonized;
+
 void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
 		 unsigned int pktlen, char *message);
 void write_tcp_unclosed(int logging, FILE * fd, struct tcptable *table);
@@ -63,7 +66,7 @@ void ipmonhelp()
 {
     move(LINES - 1, 1);
     printkeyhelp("Up/Dn/PgUp/PgDn", "-scrl actv win  ", stdscr);
-    move(LINES-1, 50);
+    move(LINES - 1, 50);
     printkeyhelp("W", "-chg actv win  ", stdscr);
     stdexitkeyhelp();
 };
@@ -72,11 +75,11 @@ void uniq_help(int what)
 {
     move(LINES - 1, 32);
     if (!what)
-        printkeyhelp("M", "-more TCP info  ", stdscr);
+	printkeyhelp("M", "-more TCP info   ", stdscr);
     else
-        printkeyhelp("Lft/Rt", "-vert scrl ", stdscr);
-}  
-   
+	printkeyhelp("Lft/Rt", "-vert scrl  ", stdscr);
+}
+
 /* Mark general packet count indicators */
 
 void prepare_ip_statwin(WINDOW * win)
@@ -248,7 +251,6 @@ void pagelowerwin(struct othptable *table, int direction)
 int checkrvnamed(void)
 {
     int execstat = 0;
-    int resp;
     pid_t cpid = 0;
     int cstat;
     extern int errno;
@@ -267,7 +269,7 @@ int checkrvnamed(void)
 
 	    _exit(1);
 	} else if (cpid == -1) {
-	    errbox("Can't spawn new process; lookups will block", ANYKEY_MSG, &resp);
+	    write_error("Can't spawn new process; lookups will block", daemonized);
 	    return 0;
 	} else {
 	    while (waitpid(cpid, &cstat, 0) < 0)
@@ -275,7 +277,7 @@ int checkrvnamed(void)
 		    break;
 
 	    if (WEXITSTATUS(cstat) == 1) {
-		errbox("Can't start rvnamed; lookups will block", ANYKEY_MSG, &resp);
+		write_error("Can't start rvnamed; lookups will block", daemonized);
 		return 0;
 	    } else {
 		sleep(1);
@@ -292,15 +294,15 @@ int checkrvnamed(void)
 
 void ipmon(const struct OPTIONS *options, int filtered,
 	   struct filterlist *fl, struct othpoptions *ofilter,
-	   unsigned int tcptm, int facilitytime)
+	   unsigned int tcptm, int facilitytime, char *ifptr)
 {
     int logging = options->logging;
     struct sockaddr_ll fromaddr;	/* iface info */
     unsigned short linktype;	/* data link type */
     int fd;			/* raw socket */
     int isdn_fd = -1;		/* file descriptor for isdnctrl if any */
-    char tpacket[8194];		/* raw packet data */
-    char aligned_buf[120];	/* aligned buffer */
+    char tpacket[MAX_PACKET_SIZE];	/* raw packet data */
+    char aligned_buf[ALIGNED_BUF_LEN];	/* aligned buffer */
     char *packet = NULL;	/* network packet ptr */
     struct iphdr *ippacket;
     struct tcphdr *transpacket;	/* IP-encapsulated packet */
@@ -360,7 +362,6 @@ void ipmon(const struct OPTIONS *options, int filtered,
     int fragment = 0;		/* Set to 1 if not first fragment */
 
     int ch;
-    int endloop = 0;
     char msgstring[40];
     int nomem = 0;
 
@@ -373,13 +374,26 @@ void ipmon(const struct OPTIONS *options, int filtered,
     if (!facility_active(IPMONIDFILE))
 	mark_facility(IPMONIDFILE, "IP traffic monitor");
     else {
-	errbox("IP traffic monitor active in another process", ANYKEY_MSG, &ch);
+	write_error("IP Traffic Monitor active in another process", daemonized);
 	return;
     }
 
+    if (ifptr != NULL) {
+	if (!iface_supported(ifptr)) {
+	    err_iface_unsupported();
+	    unlink(IPMONIDFILE);
+	    return;
+	}
+	if (!iface_up(ifptr)) {
+	    err_iface_down();
+	    unlink(IPMONIDFILE);
+	    return;
+	}
+    }
     open_socket(&fd);
 
     if (fd < 0) {
+	unlink(IPMONIDFILE);
 	return;
     }
     init_tcp_table(&table);
@@ -429,10 +443,11 @@ void ipmon(const struct OPTIONS *options, int filtered,
     rotate_flag = 0;
     writelog(logging, logfile, "******** IP traffic monitor started");
 
+    exitloop = 0;
     gettimeofday(&tv, NULL);
     starttime = timeint = tv.tv_sec;
 
-    while (!endloop) {
+    while (!exitloop) {
 	getpacket(fd, tpacket, &fromaddr, &ch, &readlen, ifname, table.tcpscreen);
 
 	if (ch != ERR) {
@@ -450,21 +465,21 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		    scrolllowerwin(&othptbl, SCROLLUP);
 		break;
 	    case KEY_RIGHT:
-	        if (curwin) {
-	            if (othptbl.strindex != VSCRL_OFFSET)
-	                othptbl.strindex = VSCRL_OFFSET;
-	            
-	            refresh_othwindow(&othptbl);
-	        }
-	        break;
+		if (curwin) {
+		    if (othptbl.strindex != VSCRL_OFFSET)
+			othptbl.strindex = VSCRL_OFFSET;
+
+		    refresh_othwindow(&othptbl);
+		}
+		break;
 	    case KEY_LEFT:
-	        if (curwin) {
-	            if (othptbl.strindex != 0)
-	                othptbl.strindex = 0;
-	            
-	            refresh_othwindow(&othptbl);
-	        }
-	        break;
+		if (curwin) {
+		    if (othptbl.strindex != 0)
+			othptbl.strindex = 0;
+
+		    refresh_othwindow(&othptbl);
+		}
+		break;
 	    case KEY_PPAGE:
 	    case '-':
 		if (!curwin)
@@ -488,24 +503,36 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		break;
 	    case 'm':
 	    case 'M':
-	        if (!curwin) {
+		if (!curwin) {
 		    mode = ~mode;
 		    refreshtcpwin(&table, screen_idx, mode);
 		}
 		break;
+	    case 12:
+	    case 'l':
+	    case 'L':
+		refresh_screen();
+		break;
+
 	    case 'Q':
 	    case 'q':
 	    case 'X':
 	    case 'x':
 	    case 24:
 	    case 27:
-		endloop = 1;
+		exitloop = 1;
 	    }
 	}
 	if (readlen > 0) {
 	    if (!iface_supported(ifname))
 		continue;
 
+	    /* Is is for any particular interface only? */
+
+	    if (ifptr != NULL) {
+		if (strcmp(ifname, ifptr) != 0)
+		    continue;
+	    }
 	    isdn_iface_check(&isdn_fd, ifname);
 	    linktype = getlinktype(fromaddr.sll_hatype, ifname,
 				   isdn_fd, &isdntable);
@@ -519,7 +546,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		    nifltok = othfilterok(ofilter, fromaddr.sll_protocol,
 					  0, 0, 0, 0);
 		else
-		    nifltok = 1;
+		    nifltok = othfilterok(ofilter, 0, 0, 0, 0, 0);
 
 
 		/*
@@ -757,7 +784,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 	    updtime_usec = unow;
 	}
 	if ((facilitytime != 0) && (((now - starttime) / 60) >= facilitytime))
-	    endloop = 1;
+	    exitloop = 1;
 
 	if ((rotate_flag == 1) && (logging)) {
 	    announce_rotate_prepare(logfile);
