@@ -1,4 +1,3 @@
-
 /***
 
 ethdesc.c	- Ethernet host description management module
@@ -17,21 +16,7 @@ GNU General Public License for more details.
 
 ***/
 
-struct descrec {
-    char address[13];
-    char desc[65];
-};
-
-struct desclistent {
-    struct descrec rec;
-    struct desclistent *prev_entry;
-    struct desclistent *next_entry;
-};
-
-struct desclist {
-    struct desclistent *head;
-    struct desclistent *tail;
-};
+#include "descrec.h"
 
 void etherr(void);
 void loaddesclist(struct desclist *list, unsigned int linktype);
