@@ -36,8 +36,12 @@ details.
 #include "mode.h"
 #include "options.h"
 
+#define MSGSTRING_MAX		240
+#define TARGET_LOGNAME_MAX	160
+#define TIME_TARGET_MAX		30
+
 volatile int rotate_flag;
-char target_logname[80];
+char target_logname[TARGET_LOGNAME_MAX];
 
 void openlogerr()
 {
@@ -64,7 +68,7 @@ void genatime(time_t now, char *atime)
 
 void writelog(int logging, FILE * fd, char *msg)
 {
-    char atime[28];
+    char atime[TIME_TARGET_MAX];
 
     if (logging) {
 	genatime(time((time_t *) NULL), atime);
@@ -72,10 +76,21 @@ void writelog(int logging, FILE * fd, char *msg)
     }
 }
 
+void write_daemon_err(char *msg)
+{
+    char atime[TIME_TARGET_MAX];
+    FILE *fd;
+
+    genatime(time((time_t *) NULL), atime);
+    fd = fopen(DAEMONLOG, "a");
+    fprintf(fd, "%s: %s\n", atime, msg);
+    fclose(fd);
+}
+
 void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
 		 unsigned int pktlen, char *message)
 {
-    char msgbuf[240];
+    char msgbuf[MSGSTRING_MAX];
 
     if (logging) {
 	sprintf(msgbuf, "TCP; %s; %u bytes; from %s:%s to %s:%s; %s",
@@ -88,7 +103,7 @@ void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
 
 void write_tcp_unclosed(int logging, FILE * fd, struct tcptable *table)
 {
-    char msgbuf[160];
+    char msgbuf[MSGSTRING_MAX];
 
     struct tcptableent *entry = table->head;
 
@@ -109,8 +124,8 @@ void writeothplog(int logging, FILE * fd, char *protname, char *description,
 		  char *additional, int is_ip, int withmac,
 		  struct othptabent *entry)
 {
-    char msgbuffer[240];
-    char scratchpad[160];
+    char msgbuffer[MSGSTRING_MAX];
+    char scratchpad[MSGSTRING_MAX];
 
     if (logging) {
 	bzero(msgbuffer, 240);
@@ -149,7 +164,7 @@ void writeothplog(int logging, FILE * fd, char *protname, char *description,
 void writegstatlog(struct iftab *table, int unit, unsigned long nsecs, FILE * fd)
 {
     struct iflist *ptmp = table->head;
-    char atime[26];
+    char atime[TIME_TARGET_MAX];
     char unitstring[7];
 
     genatime(time((time_t *) NULL), atime);
@@ -187,7 +202,7 @@ void writedstatlog(char *ifname, int unit, float activity, float pps,
 		   struct iftotals *ts, unsigned long nsecs,
 		   FILE * fd)
 {
-    char atime[26];
+    char atime[TIME_TARGET_MAX];
     char unitstring[7];
 
     dispmode(unit, unitstring);
@@ -257,7 +272,7 @@ void writedstatlog(char *ifname, int unit, float activity, float pps,
 
 void writeutslog(struct portlistent *list, unsigned long nsecs, FILE * fd)
 {
-    char atime[26];
+    char atime[TIME_TARGET_MAX];
     struct portlistent *ptmp = list;
 
     genatime(time((time_t *) NULL), atime);
@@ -282,7 +297,7 @@ void writeutslog(struct portlistent *list, unsigned long nsecs, FILE * fd)
 
 void writeethlog(struct ethtabent *list, int unit, unsigned long nsecs, FILE * fd)
 {
-    char atime[26];
+    char atime[TIME_TARGET_MAX];
     struct ethtabent *ptmp = list;
     char unitstring[7];
 
@@ -336,7 +351,7 @@ void writeethlog(struct ethtabent *list, int unit, unsigned long nsecs, FILE * f
 void write_size_log(struct ifstat_brackets *brackets, unsigned long nsecs,
 		    char *ifname, unsigned int mtu, FILE * logfile)
 {
-    char atime[26];
+    char atime[TIME_TARGET_MAX];
     int i;
 
     genatime(time((time_t *) NULL), atime);
