@@ -45,7 +45,7 @@ void makeoptionmenu(struct MENU *menu)
     additem(menu, " ^R^everse DNS lookups", "Toggles resolution of IP addresses into host names");
     additem(menu, " TCP/UDP ^s^ervice names", "Displays TCP/UDP service names instead of numeric ports");
     additem(menu, " ^P^romiscuous operation", "Toggles capture of all packets by LAN interfaces");
-    additem(menu, " ^C^olor", "Turns color on or off (restart to effect change)");
+    additem(menu, " ^C^olor", "Turns color on or off (restart IPTraf to effect change)");
     additem(menu, " ^L^ogging", "Toggles logging of traffic to a data file");
     additem(menu, " Acti^v^ity mode", "Toggles activity indicators between kbits/s and kbytes/s");
     additem(menu, NULL, NULL);
