@@ -9,14 +9,15 @@ Copyright (c) Gerard Paul Java 1997
 
 #include <sys/types.h>
 #include <netinet/udp.h>
+#include "utfilter.h"
 
 struct othpoptions {
-    char filename[40];		/* these are for the UDP filters */
+    char filename[FLT_FILENAME_MAX];	/* these are for the UDP filters */
     struct filterlist fl;
     unsigned int filtercode;
 
     u_int16_t udp:1, icmp:1, ospf:1, igp:1, igmp:1, igrp:1, arp:1, rarp:1,
-     padding:8;
+     nonip:1, padding:7;
 };
 
 void othfilterselect(struct othpoptions *filter);
