@@ -29,23 +29,26 @@ Copyright (c) Gerard Paul Java 1997
 #define TIND 1           /* Tail indicator printed */
 #define HIND 2		 /* Head indicator printed */
 
+#define VSCRL_OFFSET	30
+
 struct othptabent {
     unsigned long int saddr;
     unsigned long int daddr;
     char smacaddr[15];
     char dmacaddr[15];
     unsigned short linkproto;
-    char s_fqdn[16];
-    char d_fqdn[16];
+    char s_fqdn[100];
+    char d_fqdn[100];
     int s_fstat;
     int d_fstat;
     unsigned int protocol;
     char iface[8];
+    unsigned int pkt_length;
   
     union {
 	struct {
-	    char s_sname[11];
-	    char d_sname[11];
+	    char s_sname[15];
+	    char d_sname[15];
 	} udp;
 	struct {
 	    unsigned int type;
@@ -56,11 +59,22 @@ struct othptabent {
 	    unsigned long area;
 	    char routerid[16];
 	} ospf;
+	struct {
+	    unsigned short opcode;
+	    char src_ip_address[4];
+	    char dest_ip_address[4];
+	} arp;
+	struct {
+	    unsigned short opcode;
+	    char src_mac_address[6];
+	    char dest_mac_address[6];
+	} rarp;
     } un;
     unsigned int type;
     unsigned int code;
     unsigned int index;
     int is_ip;
+    int fragment;
     struct othptabent *prev_entry;
     struct othptabent *next_entry;
 };
@@ -72,6 +86,7 @@ struct othptable {
     struct othptabent *lastvisible;
     unsigned int count;
     unsigned int lastpos;
+    unsigned int strindex;      /* starting index of the string to display */
     int htstat;
     unsigned int obmaxy;	/* number of lines in the border window */
     unsigned int oimaxy;        /* number of lines inside the border */
@@ -81,6 +96,14 @@ struct othptable {
     PANEL *borderpanel;
 };
 
+/* Added by David Harbaugh for Non-IP protocol identification */
+
+struct packetstruct {
+    char *packet_name;  	/* Name of packet type   */
+    unsigned int protocol;      /* Number of packet type */
+ };
+
+
 /* partially stolen from ospf.h from tcpdump */
 
 #define	OSPF_TYPE_UMD	0
@@ -111,12 +134,19 @@ struct othptabent *add_othp_entry(struct othptable *table,
 				  unsigned long saddr,
 				  unsigned long daddr, int is_ip,
 				  int protocol, unsigned short linkproto,
-				  char *packet, char *ifname,
+				  char *packet,char *netpacket,
+				  unsigned int br, char *ifname,
 				  int rev_lookup, int rvnamedon, 
 				  unsigned int tm, int logging,
-				  FILE * logfile, int servnames, int *nomem);
+				  FILE * logfile, int servnames, int fragment,
+				  int *nomem);
+
+char *packetlookup(unsigned int protocol);
 
 void printothpentry(struct othptable *table, struct othptabent *entry,
-		    unsigned int screen_idx, int logging, FILE * logfile);
+		    unsigned int screen_idx, int logging,
+		    FILE * logfile);
+
+void refresh_othwindow(struct othptable *table);
 
 void destroyothptable(struct othptable *table);
