@@ -44,6 +44,9 @@ details.
 #include "log.h"
 #include "logvars.h"
 
+extern int exitloop;
+extern int daemonized;
+
 extern void write_size_log(struct ifstat_brackets *brackets,
 			   unsigned long interval, char *ifname,
 			   unsigned int mtu, FILE * logfile);
@@ -60,7 +63,6 @@ int initialize_brackets(char *ifname, struct ifstat_brackets *brackets,
 {
     struct ifreq ifr;
     int fd;
-    int resp;
     int istat;
     int i;
 
@@ -69,14 +71,14 @@ int initialize_brackets(char *ifname, struct ifstat_brackets *brackets,
     fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
 
     if (fd < 0) {
-	errbox("Unable to open socket for MTU determination", ANYKEY_MSG, &resp);
+	write_error("Unable to open socket for MTU determination", daemonized);
 	return 1;
     }
     istat = ioctl(fd, SIOCGIFMTU, &ifr);
 
     close(fd);
     if (istat < 0) {
-	errbox("Unable to obtain interface MTU", ANYKEY_MSG, &resp);
+	write_error("Unable to obtain interface MTU", daemonized);
 	return 1;
     }
     *interval = ifr.ifr_mtu / 20;	/* There are 20 packet size brackets */
@@ -148,11 +150,10 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
     unsigned int interval;
 
     int ch;
-    int resp;
 
     int fd;
 
-    char buf[8192];
+    char buf[MAX_PACKET_SIZE];
     int br;
     char iface[10];
     unsigned int mtu;
@@ -166,15 +167,13 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
     unsigned long updtime = 0;
     unsigned long long updtime_usec = 0;
 
-    int exitloop = 0;
-
     int logging = options->logging;
     FILE *logfile = NULL;
 
     if (!facility_active(PKTSIZEIDFILE))
 	mark_facility(PKTSIZEIDFILE, "Packet size breakdown");
     else {
-	errbox("Packet size breakdown already active in another process", ANYKEY_MSG, &resp);
+	write_error("Packet size breakdown already active in another process", daemonized);
 	return;
     }
 
@@ -231,6 +230,7 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
 
     writelog(logging, logfile, "******** Packet size distribution facility started");
 
+    exitloop = 0;
     gettimeofday(&tv, NULL);
     starttime = startlog = timeint = tv.tv_sec;
 
@@ -241,6 +241,11 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
 
 	if (ch != ERR) {
 	    switch (ch) {
+	    case 12:
+	    case 'l':
+	    case 'L':
+		refresh_screen();
+		break;
 	    case 'x':
 	    case 'X':
 	    case 'q':
@@ -250,9 +255,9 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname, int facilityti
 		exitloop = 1;
 	    }
 	}
-	if (br > 0) {
+	if ((br > 0) && (strcmp(iface, ifname) == 0))
 	    update_size_distrib(br, brackets, interval, win);
-	}
+	
 	gettimeofday(&tv, NULL);
 	now = tv.tv_sec;
 	unow = tv.tv_sec * 1e+6 + tv.tv_usec;
