@@ -33,6 +33,8 @@ details.
 #include "ifstats.h"
 #include "error.h"
 
+#define PROMISC_MSG_MAX 80
+
 extern void initiflist(struct iflist **list);
 extern void destroyiflist(struct iflist *list);
 
@@ -45,7 +47,7 @@ void srpromisc(int mode)
     int istat;
     int response;
     char *errmsg = "Promisc change failed for ";
-    char fullmsg[40];
+    char fullmsg[PROMISC_MSG_MAX];
 
     initiflist(&list);
     ptmp = list;
