@@ -67,12 +67,14 @@ int rvnamedactive()
 
     do {
 	sstat = select(fd + 1, &sockset, NULL, NULL, &tv);
-    } while ((sstat < 0) && (errno != ENOMEM));
+    } while ((sstat < 0) && (errno != ENOMEM) && (errno == EINTR));
 
     if (sstat == 1) {
 	fr = sizeof(su.sun_family) + strlen(su.sun_path);
-	br = recvfrom(fd, &rpkt, sizeof(struct rvn), 0, (struct sockaddr *) &su,
-		      &fr);
+	do {
+	    br = recvfrom(fd, &rpkt, sizeof(struct rvn), 0, (struct sockaddr *) &su,
+			  &fr);
+	} while ((br < 0) && (errno == EINTR));
 
 	if (br < 0)
 	    printipcerr();
@@ -133,7 +135,7 @@ int revname(int lookup, struct in_addr *saddr, char *target, int rvnfd)
     struct sockaddr_un su;
     int fl;
 
-    bzero(target, 16);
+    bzero(target, 45);
     if (lookup) {
 	if (rvnfd > 0) {
 	    su.sun_family = AF_UNIX;
@@ -146,13 +148,17 @@ int revname(int lookup, struct in_addr *saddr, char *target, int rvnfd)
 		   sizeof(su.sun_family) + strlen(su.sun_path));
 
 	    fl = sizeof(su.sun_family) + strlen(su.sun_path);
-	    br = recvfrom(rvnfd, &rpkt, sizeof(struct rvn), 0, (struct sockaddr *) &su,
-			  &fl);
+	    do {
+		br = recvfrom(rvnfd, &rpkt, sizeof(struct rvn), 0, (struct sockaddr *) &su,
+			      &fl);
+	    } while ((br < 0) && (errno == EINTR));
 
-	    if (br < 0)
+	    if (br < 0) {
+		strcpy(target, inet_ntoa(*saddr));
 		printipcerr();
-
-	    strncpy(target, rpkt.fqdn, 15);
+		return RESOLVED;
+	    }
+	    strncpy(target, rpkt.fqdn, 44);
 	    return (rpkt.ready);
 	} else {
 	    he = gethostbyaddr((char *) saddr,
@@ -161,7 +167,7 @@ int revname(int lookup, struct in_addr *saddr, char *target, int rvnfd)
 	    if (he == NULL)
 		strcpy(target, inet_ntoa(*saddr));
 	    else
-		strncpy(target, he->h_name, 15);
+		strncpy(target, he->h_name, 44);
 
 	    return RESOLVED;
 	}
