@@ -49,6 +49,7 @@ details.
 #include "dirs.h"
 
 #define NUM_CACHE_ENTRIES 2048
+#define TIME_TARGET_MAX 30
 
 struct hosts {
     unsigned long addr;
@@ -142,7 +143,7 @@ int addrstat(struct rvn *rvnpacket, struct hosts *hostlist, unsigned int lastfre
 void writervnlog(FILE * fd, char *msg)
 {
     time_t now;
-    char atime[26];
+    char atime[TIME_TARGET_MAX];
 
     now = time((time_t *) NULL);
 
