@@ -56,6 +56,9 @@ details.
 #define SCROLLUP 0
 #define SCROLLDOWN 1
 
+extern int exitloop;
+extern int daemonized;
+
 extern void writeutslog(struct portlistent *list, unsigned long nsecs, FILE * logfile);
 
 /*
@@ -332,8 +335,8 @@ void servmon(char *ifname, struct porttab *ports,
     int fd;
     int isdn_fd = -1;
 
-    char buf[8192];
-    char aligned_buf[120];
+    char buf[MAX_PACKET_SIZE];
+    char aligned_buf[ALIGNED_BUF_LEN];
     char *ipacket;
     char *tpacket;
 
@@ -341,7 +344,6 @@ void servmon(char *ifname, struct porttab *ports,
     struct sockaddr_ll fromaddr;
     unsigned short linktype;
     int br;
-    int exitloop = 0;
 
     char iface[8];
     unsigned int idx = 1;
@@ -375,7 +377,7 @@ void servmon(char *ifname, struct porttab *ports,
     if (!facility_active(TCPUDPIDFILE))
 	mark_facility(TCPUDPIDFILE, "TCP/UDP monitor");
     else {
-	errbox("TCP/UDP monitor already active in another process", ANYKEY_MSG, &ch);
+	write_error("TCP/UDP monitor already active in another process", daemonized);
 	return;
     }
 
@@ -421,10 +423,11 @@ void servmon(char *ifname, struct porttab *ports,
 
     bzero(&isdntable, sizeof(struct isdntab));
 
+    exitloop = 0;
     gettimeofday(&tv, NULL);
     starttime = startlog = timeint = tv.tv_sec;
 
-    do {
+    while (!exitloop) {
 	getpacket(fd, buf, &fromaddr, &ch, &br, iface, list.win);
 
 	if (ch != ERR)
@@ -443,6 +446,11 @@ void servmon(char *ifname, struct porttab *ports,
 	    case ' ':
 		pageservwin(&list, SCROLLUP, &idx);
 		break;
+	    case 12:
+	    case 'l':
+	    case 'L':
+		refresh_screen();
+		break;
 	    case 'q':
 	    case 'Q':
 	    case 'x':
@@ -527,7 +535,7 @@ void servmon(char *ifname, struct porttab *ports,
 
 	if ((facilitytime != 0) && (((now - starttime) / 60) >= facilitytime))
 	    exitloop = 1;
-    } while (!exitloop);
+    }
 
     if (logging) {
 	signal(SIGUSR1, SIG_DFL);
@@ -763,7 +771,7 @@ void operate_portselect(struct porttab **table, struct porttab **node,
     int ch = 0;
     int i = 0;
 
-    int exitloop = 0;
+    int endloop = 0;
 
     *node = *table;
 
@@ -810,20 +818,25 @@ void operate_portselect(struct porttab **table, struct porttab **node,
 	    }
 	    break;
 	case 13:
-	    exitloop = 1;
+	    endloop = 1;
 	    *aborted = 0;
 	    break;
+	case 12:
+	case 'l':
+	case 'L':
+	    refresh_screen();
+	    break;
 	case 27:
 	case 24:
 	case 'x':
 	case 'X':
 	case 'q':
 	case 'Q':
-	    exitloop = 1;
+	    endloop = 1;
 	    *aborted = 1;
 	    break;
 	}
-    } while (!exitloop);
+    } while (!endloop);
 }
 
 void selectport(struct porttab **table, struct porttab **node, int *aborted)
