@@ -25,6 +25,8 @@ details.
 #include "revname.h"
 #include "rvnamed.h"
 
+#define MSGSTRING_MAX	255
+
 unsigned int bmaxy = 0;
 unsigned int imaxy = 0;
 
@@ -95,7 +97,7 @@ void init_tcp_table(struct tcptable *table)
     table->firstvisible = table->lastvisible = NULL;
     table->tcpscreen = newwin(table->imaxy, COLS - 2, 2, 1);
     table->tcppanel = new_panel(table->tcpscreen);
-    table->closedentries = NULL;
+    table->closedentries = table->closedtail = NULL;
     wattrset(table->tcpscreen, BOXATTR);
     colorwin(table->tcpscreen);
     table->lastpos = 0;
@@ -263,6 +265,15 @@ struct tcptableent *addentry(struct tcptable *table,
 	ctemp = table->closedentries;
 	table->closedentries = table->closedentries->next_entry;
 	free(ctemp);
+
+	/*
+	 * Mark the closed list's tail as NULL if we use the last entry
+	 * in the list to prevent a dangling reference.
+	 */
+
+	if (table->closedentries == NULL)
+	    table->closedtail = NULL;
+
 	new_entry->reused = new_entry->oth_connection->reused = 1;
 
 	/*
@@ -362,11 +373,26 @@ void addtoclosedlist(struct tcptable *table, struct tcptableent *entry,
 	*nomem = 1;
 	return;
     }
+    /*
+     * Point to closed entries
+     */
     ctemp->closedentry = entry;
     ctemp->pair = entry->oth_connection;
-    ctemp->next_entry = table->closedentries;
     entry->inclosed = entry->oth_connection->inclosed = 1;
-    table->closedentries = ctemp;
+
+    /*
+     * Add node to closed entry list.
+     */
+
+    if (table->closedtail != NULL)
+	table->closedtail->next_entry = ctemp;
+
+    table->closedtail = ctemp;
+    table->closedtail->next_entry = NULL;
+
+    if (table->closedentries == NULL)
+	table->closedentries = ctemp;
+
 }
 
 struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
@@ -378,7 +404,7 @@ struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
     struct tcp_hashentry *hashptr;
     unsigned int hp;
 
-    char msgstring[240];
+    char msgstring[MSGSTRING_MAX];
 
     int hastimeouts = 0;
 
@@ -448,7 +474,7 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 		 unsigned int bcount, unsigned int fragofs, int logging,
 		 int revlook, int rvnfd, FILE * logfile, int *nomem)
 {
-    char msgstring[160];
+    char msgstring[MSGSTRING_MAX];
 
     if (tableentry->s_fstat != RESOLVED) {
 	tableentry->s_fstat = revname(revlook, &(tableentry->saddr),
@@ -625,7 +651,7 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 {
     char stat[7] = "";
     unsigned int target_row;
-    char sp_buf[255];
+    char sp_buf[MSGSTRING_MAX];
 
     if ((tableentry->index < screen_idx) ||
 	(tableentry->index > screen_idx + (table->imaxy - 1)))
