@@ -6,22 +6,7 @@ Copyright (c) Gerard Paul Java 1997
 
 ***/
 
-#define F_TCP 0
-#define F_UDP 1
-
-struct filterfileent {
-    char desc[35];
-    char filename[40];
-};
-
-struct hostparams {
-    char s_fqdn[25];
-    char d_fqdn[25];
-    char s_mask[20];
-    char d_mask[20];
-    unsigned int sport;
-    unsigned int dport;
-};
+#include "fltmgr.h"
 
 struct filterent {
     struct hostparams hp;
@@ -39,14 +24,8 @@ struct filterlist {
     struct filterent *tail;
     unsigned int lastpos;
 
-    WINDOW *borderwin;
+/*    WINDOW *borderwin;
     PANEL *borderpanel;
     WINDOW *filterwin;
-    PANEL *filterpanel;
-};
-
-struct ffnode {
-    struct filterfileent ffe;
-    struct ffnode *next_entry;
-    struct ffnode *prev_entry;
+    PANEL *filterpanel; */
 };
