@@ -30,11 +30,13 @@ details.
 #include <netdb.h>
 #include <ctype.h>
 #include "dirs.h"
+#include "getpath.h"
 #include "deskman.h"
 #include "attrs.h"
 #include "menurt.h"
 #include "utfdefs.h"
 #include "utfilter.h"
+#include "fltedit.h"
 #include "input.h"
 #include "error.h"
 #include "instances.h"
@@ -58,7 +60,10 @@ void listfileerr(int code)
 	errbox("Error writing TCP/UDP filter list file", ANYKEY_MSG, &resp);
 }
 
-void gethostparams(struct hostparams *hp, int *aborted)
+void gethostparams(struct hostparams *hp, int *aborted,
+		   char *initsrc, char *initsmask, char *initsport,
+		   char *initdest, char *initdmask, char *initdport,
+		   char *initinex)
 {
     struct FIELDLIST fieldlist;
     struct FIELD *dlist;
@@ -95,13 +100,13 @@ void gethostparams(struct hostparams *hp, int *aborted)
 
     initfields(&fieldlist, 8, 52, (LINES - 12) / 2 + 1, (COLS - 80) / 2 + 27);
 
-    addfield(&fieldlist, 25, 1, 0, "");
-    addfield(&fieldlist, 25, 3, 0, "");
-    addfield(&fieldlist, 5, 5, 0, "");
-    addfield(&fieldlist, 25, 1, 26, "0.0.0.0");
-    addfield(&fieldlist, 25, 3, 26, "0.0.0.0");
-    addfield(&fieldlist, 5, 5, 26, "0");
-    addfield(&fieldlist, 1, 7, 0, "I");
+    addfield(&fieldlist, 25, 1, 0, initsrc);
+    addfield(&fieldlist, 25, 3, 0, initsmask);
+    addfield(&fieldlist, 5, 5, 0, initsport);
+    addfield(&fieldlist, 25, 1, 26, initdest);
+    addfield(&fieldlist, 25, 3, 26, initdmask);
+    addfield(&fieldlist, 5, 5, 26, initdport);
+    addfield(&fieldlist, 1, 7, 0, initinex);
 
     dlist = fieldlist.list->nextfield->nextfield->nextfield;
 
@@ -158,20 +163,13 @@ void clear_flt_tag(int protocol)
 
 void definefilter(int protocol, int *aborted)
 {
-    struct FIELDLIST descfield;
     struct filterfileent ffile;
     struct hostparams hp;
-    char ffilename[40] = WORKDIR;
     char fntemp[14];
 
     int pfd;
     int bw;
     int resp;
-    
-    int dlgwintop;
-
-    WINDOW *dlgwin;
-    PANEL *dlgpanel;
 
     /*
      * Lock facility
@@ -179,39 +177,16 @@ void definefilter(int protocol, int *aborted)
 
     if (!mark_filter_change(protocol))
 	return;
-    
-    dlgwintop = (LINES - 9) / 2;
-    dlgwin = newwin(7, 42, dlgwintop, (COLS - 42) / 2 - 10);
-    dlgpanel = new_panel(dlgwin);
-    wattrset(dlgwin, BOXATTR);
-    colorwin(dlgwin);
-    box(dlgwin, ACS_VLINE, ACS_HLINE);
-    wattrset(dlgwin, STDATTR);
-    wmove(dlgwin, 2, 2);
-    wprintw(dlgwin, "Enter a description for this filter");
-    wmove(dlgwin, 5, 2);
-    stdkeyhelp(dlgwin);
-    update_panels();
-    doupdate();
 
-    initfields(&descfield, 1, 35, dlgwintop + 3, (COLS - 42) / 2 - 8);
-    addfield(&descfield, 33, 0, 0, "");
-    fillfields(&descfield, aborted);
-    strcpy(ffile.desc, descfield.list->buf);
-    destroyfields(&descfield);
-    del_panel(dlgpanel);
-    delwin(dlgwin);
-    update_panels();
-    doupdate();
+    get_filter_description(ffile.desc, aborted, "");
 
     if (*aborted) {
 	clear_flt_tag(protocol);
 	return;
     }
     genname(time((time_t *) NULL), fntemp);
-    strcat(ffilename, fntemp);
 
-    pfd = open(ffilename, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR);
+    pfd = open(get_path(T_WORKDIR, fntemp), O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR);
     if (pfd < 0) {
 	errbox("Cannot create TCP/UDP filter record file", ANYKEY_MSG, &resp);
 	*aborted = 1;
@@ -219,7 +194,7 @@ void definefilter(int protocol, int *aborted)
 	return;
     }
     do {
-	gethostparams(&hp, aborted);
+	gethostparams(&hp, aborted, "", "", "", "0.0.0.0", "0.0.0.0", "0", "I");
 	if (!(*aborted)) {
 	    bw = write(pfd, &hp, sizeof(struct hostparams));
 
@@ -330,6 +305,11 @@ void operate_select(struct ffnode *ffiles, WINDOW * win, struct ffnode **item,
 		pptr = pptr->next_entry;
 	    }
 	    break;
+	case 12:
+	case 'l':
+	case 'L':
+	    refresh_screen();
+	    break;
 	case 13:
 	    exitloop = 1;
 	    *aborted = 0;
@@ -430,9 +410,11 @@ unsigned long int nametoaddr(char *ascname)
     return result;
 }
 
-/* loads the filter from the filter file */
+/* 
+ * Loads the filter from the filter file
+ */
 
-void loadfilter(char *filename, struct filterlist *fl)
+int loadfilter(char *filename, struct filterlist *fl, int resolve)
 {
     struct filterent *fe;
     int pfd;
@@ -447,7 +429,7 @@ void loadfilter(char *filename, struct filterlist *fl)
     if (pfd < 0) {
 	errbox("Error opening TCP/UDP filter file", ANYKEY_MSG, &resp);
 	fl->head = NULL;
-	return;
+	return 1;
     }
     do {
 	fe = malloc(sizeof(struct filterent));
@@ -455,16 +437,17 @@ void loadfilter(char *filename, struct filterlist *fl)
 
 	if (br > 0) {
 	    fe->index = idx;
-	    fe->saddr = nametoaddr(fe->hp.s_fqdn);
-	    fe->daddr = nametoaddr(fe->hp.d_fqdn);
+	    if (resolve) {
+		fe->saddr = nametoaddr(fe->hp.s_fqdn);
+		fe->daddr = nametoaddr(fe->hp.d_fqdn);
 
-	    if ((fe->saddr == -1) || (fe->daddr == -1)) {
-		free(fe);
-		continue;
+		if ((fe->saddr == -1) || (fe->daddr == -1)) {
+		    free(fe);
+		    continue;
+		}
+		fe->smask = inet_addr(fe->hp.s_mask);
+		fe->dmask = inet_addr(fe->hp.d_mask);
 	    }
-	    fe->smask = inet_addr(fe->hp.s_mask);
-	    fe->dmask = inet_addr(fe->hp.d_mask);
-
 	    if (fl->head == NULL) {
 		fl->head = fe;
 		fe->prev_entry = NULL;
@@ -482,6 +465,30 @@ void loadfilter(char *filename, struct filterlist *fl)
 
     if (br == 0)
 	close(pfd);
+	
+    return 0;
+}
+
+void savefilter(unsigned int protocol, char *filename, struct filterlist *fl)
+{
+    struct filterent *fe = fl->head;
+    int pfd;
+    int bw;
+    int resp;
+    
+    pfd = open(filename, O_CREAT | O_TRUNC | O_WRONLY, S_IRUSR | S_IWUSR);
+
+    while (fe != NULL) {
+	bw = write(pfd, &(fe->hp), sizeof(struct hostparams));
+	if (bw < 0) {
+	    errbox("Unable to save filter changes", ANYKEY_MSG, &resp);
+	    clear_flt_tag(protocol);
+	    return;
+	}
+	fe = fe->next_entry;
+    }
+
+    close(pfd);
 }
 
 void marktcpfilter(char *filename)
@@ -491,7 +498,7 @@ void marktcpfilter(char *filename)
 
     fd = open(TCPFILTERSAVE, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
 
-    if (write(fd, filename, 40) < 0) {
+    if (write(fd, filename, FLT_FILENAME_MAX) < 0) {
 	errbox("Unable to save TCP/UDP filter name", ANYKEY_MSG, &resp);
 
 	if (fd > 0)
@@ -505,7 +512,7 @@ void marktcpfilter(char *filename)
 void loadsavedtcpfilter(struct filterlist *fl, int *filtered)
 {
     int fd;
-    char filename[40];
+    char filename[FLT_FILENAME_MAX];
     int resp;
 
     fd = open(TCPFILTERSAVE, O_RDONLY);
@@ -513,8 +520,8 @@ void loadsavedtcpfilter(struct filterlist *fl, int *filtered)
     if (fd < 0)
 	return;
 
-    bzero(filename, 40);
-    if (read(fd, filename, 40) < 0) {
+    bzero(filename, FLT_FILENAME_MAX);
+    if (read(fd, filename, FLT_FILENAME_MAX) < 0) {
 	errbox("Unable to retrieve saved TCP/UDP filter", ANYKEY_MSG, &resp);
 	if (fd > 0)
 	    close(fd);
@@ -524,7 +531,7 @@ void loadsavedtcpfilter(struct filterlist *fl, int *filtered)
     }
     close(fd);
 
-    loadfilter(filename, fl);
+    loadfilter(filename, fl, FLT_RESOLVE);
 
     if (fl->head == NULL)
 	*filtered = 0;
@@ -590,16 +597,13 @@ void destroyfilter(struct filterlist *fl)
     }
 }
 
+
 /* delete a filter record from the disk */
 
 void delfilter(unsigned int protocol, int *aborted)
 {
     struct ffnode *fltfile;
     struct ffnode *fltlist;
-    struct ffnode *ffntemp;
-    char fntemp[40] = WORKDIR;
-    int fd;
-    int bw;
 
     if (!mark_filter_change(protocol))
 	return;
@@ -616,8 +620,7 @@ void delfilter(unsigned int protocol, int *aborted)
 	clear_flt_tag(protocol);
 	return;
     }
-    strcat(fntemp, fltfile->ffe.filename);
-    unlink(fntemp);
+    unlink(get_path(T_WORKDIR, fltfile->ffe.filename));
 
     if (fltfile->prev_entry == NULL) {
 	fltlist = fltlist->next_entry;
@@ -632,52 +635,29 @@ void delfilter(unsigned int protocol, int *aborted)
 
     free(fltfile);
 
-    if (protocol == F_TCP)
-	fd = open(TCPFLNAME, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
-    else
-	fd = open(UDPFLNAME, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
-
-    if (fd < 0) {
-	listfileerr(2);
-	clear_flt_tag(protocol);
-	return;
-    }
-    fltfile = fltlist;
-    while (fltfile != NULL) {
-	bw = write(fd, &(fltfile->ffe), sizeof(struct filterfileent));
-
-	if (bw < 0) {
-	    listfileerr(2);
-	    clear_flt_tag(protocol);
-	    return;
-	}
-	ffntemp = fltfile;
-	fltfile = fltfile->next_entry;
-	free(ffntemp);
-    }
-
-    close(fd);
-
+    save_filterlist(protocol, fltlist);
     clear_flt_tag(protocol);
 }
 
 void makefiltermenu(struct MENU *menu)
 {
-    initmenu(menu, 7, 31, (LINES - 7) / 2 + 3, (COLS - 31) / 2 + 15);
+    initmenu(menu, 8, 31, (LINES - 7) / 2 + 3, (COLS - 31) / 2 + 15);
     additem(menu, " ^D^efine new filter...", "Defines a new set of filter parameters");
     additem(menu, " ^A^pply filter...", "Applies filter to be used for TCP display");
     additem(menu, " Detac^h^ filter", "Removes the currently applied filter");
-    additem(menu, " Dele^t^e filter...", "Removes a filter from the filter list");
+    additem(menu, " ^E^dit filter...", "Modifies existing TCP filter data");
+    additem(menu, " Dele^t^e filter...", "Removes a TCP filter from the filter list");
     additem(menu, " E^x^it menu", "Returns to the main menu");
 }
 
 void makeudpfiltermenu(struct MENU *menu)
 {
-    initmenu(menu, 8, 31, (LINES - 11) / 2, (COLS - 30) / 2 - 21);
+    initmenu(menu, 9, 31, (LINES - 11) / 2, (COLS - 30) / 2 - 21);
     additem(menu, " Show ^a^ll UDP packets", "Permits display of all UDP packets");
     additem(menu, " Show ^n^o UDP packets", "Omits all UDP packets from display");
     additem(menu, " ^D^efine custom UDP filter...", "Defines a custom UDP filter");
     additem(menu, " Apply ^c^ustom UDP filter...", "Applies a custom UDP filter");
+    additem(menu, " ^E^dit custom UDP filter...", "Modifies a custom UDP filter");
     additem(menu, " Dele^t^e custom UDP filter...", "Removes a custom UDP filter");
     additem(menu, " E^x^it menu", "Returns to the main menu");
 }
@@ -717,17 +697,19 @@ void udpfilterselect(struct filterlist *fl,
 	case 4:
 	    selectfilter(F_UDP, &ffe, faborted);
 	    if (!(*faborted)) {
-		strcpy(filename, WORKDIR);
-		strcat(filename, ffe.filename);
-		loadfilter(filename, fl);
+		strncpy(filename, get_path(T_WORKDIR, ffe.filename), FLT_FILENAME_MAX - 1);
+		loadfilter(filename, fl, FLT_RESOLVE);
 		*filtercode = 2;
 	    }
 	    break;
 	case 5:
+	    editfilter(F_UDP, faborted);
+	    break;
+	case 6:
 	    delfilter(F_UDP, faborted);
 	    break;
 	}
-    } while (frow != 6);
+    } while (frow != 7);
 
     destroymenu(&fmenu);
     update_panels();
@@ -735,7 +717,9 @@ void udpfilterselect(struct filterlist *fl,
 }
 
 
-/* display a menu and perform appropriate filter operation */
+/* 
+ * Display a menu and perform appropriate filter operation
+ */
 
 void tcpfilterselect(struct filterlist *fl,
 		     unsigned int *filtered, int *faborted)
@@ -743,7 +727,7 @@ void tcpfilterselect(struct filterlist *fl,
     struct MENU fmenu;
     unsigned int frow;
     struct filterfileent ffe;
-    char filename[30];
+    char filename[FLT_FILENAME_MAX];
 
     makefiltermenu(&fmenu);
 
@@ -759,11 +743,11 @@ void tcpfilterselect(struct filterlist *fl,
 	case 2:
 	    selectfilter(F_TCP, &ffe, faborted);
 	    if (!(*faborted)) {
-		strcpy(filename, WORKDIR);
-		strcat(filename, ffe.filename);
-		loadfilter(filename, fl);
-		*filtered = 1;
-		marktcpfilter(filename);
+		strncpy(filename, get_path(T_WORKDIR, ffe.filename), FLT_FILENAME_MAX - 1);
+		if (!loadfilter(filename, fl, FLT_RESOLVE)) {
+		    *filtered = 1;
+		    marktcpfilter(filename);
+		}
 	    }
 	    break;
 	case 3:
@@ -774,10 +758,13 @@ void tcpfilterselect(struct filterlist *fl,
 	    unlink(TCPFILTERSAVE);
 	    break;
 	case 4:
+	    editfilter(F_TCP, faborted);
+	    break;
+	case 5:
 	    delfilter(F_TCP, faborted);
 	    break;
 	}
-    } while (frow != 5);
+    } while (frow != 6);
 
     destroymenu(&fmenu);
     update_panels();
